@@ -249,9 +249,10 @@ g_dbus_method_invocation_get_message (GDBusMethodInvocation *invocation)
  * g_dbus_method_invocation_get_parameters:
  * @invocation: A #GDBusMethodInvocation.
  *
- * Gets the parameters of the method invocation.
+ * Gets the parameters of the method invocation. If there are no input
+ * parameters then this will return a GVariant with 0 children rather than NULL.
  *
- * Returns: A #GVariant. Do not free, it is owned by @invocation.
+ * Returns: (transfer none): A #GVariant tuple. Do not unref this because it is owned by @invocation.
  *
  * Since: 2.26
  */
@@ -336,8 +337,8 @@ _g_dbus_method_invocation_new (const gchar           *sender,
 
 /**
  * g_dbus_method_invocation_return_value:
- * @invocation: A #GDBusMethodInvocation.
- * @parameters: A #GVariant tuple with out parameters for the method or %NULL if not passing any parameters.
+ * @invocation: (transfer full): A #GDBusMethodInvocation.
+ * @parameters: (allow-none): A #GVariant tuple with out parameters for the method or %NULL if not passing any parameters.
  *
  * Finishes handling a D-Bus method call by returning @parameters.
  * If the @parameters GVariant is floating, it is consumed.
@@ -372,8 +373,8 @@ g_dbus_method_invocation_return_value (GDBusMethodInvocation *invocation,
         {
           gchar *type_string = g_variant_type_dup_string (type);
 
-          g_warning (_("Type of return value is incorrect, got `%s', expected `%s'"),
-                     g_variant_get_type_string (parameters), type_string);
+          g_warning ("Type of return value is incorrect, got `%s', expected `%s'",
+		     g_variant_get_type_string (parameters), type_string);
           g_variant_type_free (type);
           g_free (type_string);
           goto out;
@@ -403,7 +404,7 @@ g_dbus_method_invocation_return_value (GDBusMethodInvocation *invocation,
   error = NULL;
   if (!g_dbus_connection_send_message (g_dbus_method_invocation_get_connection (invocation), reply, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, &error))
     {
-      g_warning (_("Error sending message: %s"), error->message);
+      g_warning ("Error sending message: %s", error->message);
       g_error_free (error);
     }
   g_object_unref (reply);
@@ -416,7 +417,7 @@ g_dbus_method_invocation_return_value (GDBusMethodInvocation *invocation,
 
 /**
  * g_dbus_method_invocation_return_error:
- * @invocation: A #GDBusMethodInvocation.
+ * @invocation: (transfer full): A #GDBusMethodInvocation.
  * @domain: A #GQuark for the #GError error domain.
  * @code: The error code.
  * @format: printf()-style format.
@@ -463,7 +464,7 @@ g_dbus_method_invocation_return_error (GDBusMethodInvocation *invocation,
 
 /**
  * g_dbus_method_invocation_return_error_valist:
- * @invocation: A #GDBusMethodInvocation.
+ * @invocation: (transfer full): A #GDBusMethodInvocation.
  * @domain: A #GQuark for the #GError error domain.
  * @code: The error code.
  * @format: printf()-style format.
@@ -498,7 +499,7 @@ g_dbus_method_invocation_return_error_valist (GDBusMethodInvocation *invocation,
 
 /**
  * g_dbus_method_invocation_return_error_literal:
- * @invocation: A #GDBusMethodInvocation.
+ * @invocation: (transfer full): A #GDBusMethodInvocation.
  * @domain: A #GQuark for the #GError error domain.
  * @code: The error code.
  * @message: The error message.
@@ -527,7 +528,7 @@ g_dbus_method_invocation_return_error_literal (GDBusMethodInvocation *invocation
 
 /**
  * g_dbus_method_invocation_return_gerror:
- * @invocation: A #GDBusMethodInvocation.
+ * @invocation: (transfer full): A #GDBusMethodInvocation.
  * @error: A #GError.
  *
  * Like g_dbus_method_invocation_return_error() but takes a #GError
@@ -554,9 +555,31 @@ g_dbus_method_invocation_return_gerror (GDBusMethodInvocation *invocation,
   g_free (dbus_error_name);
 }
 
+/**
+ * g_dbus_method_invocation_take_error: (skip)
+ * @invocation: (transfer full): A #GDBusMethodInvocation.
+ * @error: (transfer full): A #GError.
+ *
+ * Like g_dbus_method_invocation_return_gerror() but takes ownership
+ * of @error so the caller does not need to free it.
+ *
+ * This method will free @invocation, you cannot use it afterwards.
+ *
+ * Since: 2.30
+ */
+void
+g_dbus_method_invocation_take_error (GDBusMethodInvocation *invocation,
+                                     GError                *error)
+{
+  g_return_if_fail (G_IS_DBUS_METHOD_INVOCATION (invocation));
+  g_return_if_fail (error != NULL);
+  g_dbus_method_invocation_return_gerror (invocation, error);
+  g_error_free (error);
+}
+
 /**
  * g_dbus_method_invocation_return_dbus_error:
- * @invocation: A #GDBusMethodInvocation.
+ * @invocation: (transfer full): A #GDBusMethodInvocation.
  * @error_name: A valid D-Bus error name.
  * @error_message: A valid D-Bus error message.
  *
