@@ -33,56 +33,9 @@ G_BEGIN_DECLS
 
 #define G_TYPE_DBUS_METHOD_INVOCATION         (g_dbus_method_invocation_get_type ())
 #define G_DBUS_METHOD_INVOCATION(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), G_TYPE_DBUS_METHOD_INVOCATION, GDBusMethodInvocation))
-#define G_DBUS_METHOD_INVOCATION_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), G_TYPE_DBUS_METHOD_INVOCATION, GDBusMethodInvocationClass))
-#define G_DBUS_METHOD_INVOCATION_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), G_TYPE_DBUS_METHOD_INVOCATION, GDBusMethodInvocationClass))
 #define G_IS_DBUS_METHOD_INVOCATION(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), G_TYPE_DBUS_METHOD_INVOCATION))
-#define G_IS_DBUS_METHOD_INVOCATION_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), G_TYPE_DBUS_METHOD_INVOCATION))
-
-typedef struct _GDBusMethodInvocationClass   GDBusMethodInvocationClass;
-typedef struct _GDBusMethodInvocationPrivate GDBusMethodInvocationPrivate;
-
-/**
- * GDBusMethodInvocation:
- *
- * The #GDBusMethodInvocation structure contains only private data and
- * should only be accessed using the provided API.
- *
- * Since: 2.26
- */
-struct _GDBusMethodInvocation
-{
-  /*< private >*/
-  GObject parent_instance;
-  GDBusMethodInvocationPrivate *priv;
-};
-
-/**
- * GDBusMethodInvocationClass:
- *
- * Class structure for #GDBusMethodInvocation.
- *
- * Since: 2.26
- */
-struct _GDBusMethodInvocationClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-
-  /*< private >*/
-  /* Padding for future expansion */
-  gpointer padding[32];
-};
 
 GType                  g_dbus_method_invocation_get_type             (void) G_GNUC_CONST;
-GDBusMethodInvocation *g_dbus_method_invocation_new                  (const gchar           *sender,
-                                                                      const gchar           *object_path,
-                                                                      const gchar           *interface_name,
-                                                                      const gchar           *method_name,
-                                                                      const GDBusMethodInfo *method_info,
-                                                                      GDBusConnection       *connection,
-                                                                      GDBusMessage          *message,
-                                                                      GVariant              *parameters,
-                                                                      gpointer               user_data);
 const gchar           *g_dbus_method_invocation_get_sender           (GDBusMethodInvocation *invocation);
 const gchar           *g_dbus_method_invocation_get_object_path      (GDBusMethodInvocation *invocation);
 const gchar           *g_dbus_method_invocation_get_interface_name   (GDBusMethodInvocation *invocation);
@@ -111,6 +64,8 @@ void                   g_dbus_method_invocation_return_error_literal (GDBusMetho
                                                                       const gchar           *message);
 void                   g_dbus_method_invocation_return_gerror        (GDBusMethodInvocation *invocation,
                                                                       const GError          *error);
+void                   g_dbus_method_invocation_take_error           (GDBusMethodInvocation *invocation,
+                                                                      GError                *error);
 void                   g_dbus_method_invocation_return_dbus_error    (GDBusMethodInvocation *invocation,
                                                                       const gchar           *error_name,
                                                                       const gchar           *error_message);
