@@ -374,33 +374,33 @@ reset_all_keys (GSettings   *settings)
 
 static void
 gsettings_reset_recursively (GSettings   *settings,
-			     const gchar *key,
-			     const gchar *value)
+                             const gchar *key,
+                             const gchar *value)
 {
   gchar **children;
   gint i;
 
   g_settings_delay (settings);
-  
+
   reset_all_keys (settings);
   children = g_settings_list_children (settings);
   for (i = 0; children[i]; i++)
     {
       GSettings *child;
       gchar *schema;
-      
+
       child = g_settings_get_child (settings, children[i]);
       g_object_get (child, "schema", &schema, NULL);
-      
+
       if (is_schema (schema))
-	reset_all_keys (child);
-      
+        reset_all_keys (child);
+
       g_object_unref (child);
       g_free (schema);
     }
-  
+
   g_strfreev (children);
-  
+
   g_settings_apply (settings);
   g_settings_sync ();
 }
@@ -457,7 +457,6 @@ gsettings_set (GSettings   *settings,
   GError *error = NULL;
   GVariant *existing;
   GVariant *new;
-  GVariant *stored;
   gchar *freeme = NULL;
 
   existing = g_settings_get_value (settings, key);
@@ -465,21 +464,33 @@ gsettings_set (GSettings   *settings,
 
   new = g_variant_parse (type, value, NULL, NULL, &error);
 
-  /* A common error is to specify a string with single quotes
-   * (or use completion for that), and forget that the shell
-   * will eat one level of quoting, resulting in 'unknown keyword'
-   * error from the gvariant parser.
-   * To handle this case, try to parse again with an extra level
-   * of quotes.
+  /* If that didn't work and the type is string then we should assume
+   * that the user is just trying to set a string directly and forgot
+   * the quotes (or had them consumed by the shell).
+   *
+   * If the user started with a quote then we assume that some deeper
+   * problem is at play and we want the failure in that case.
+   *
+   * Consider:
+   *
+   *   gsettings set x.y.z key "'i don't expect this to work'"
+   *
+   * Note that we should not just add quotes and try parsing again, but
+   * rather assume that the user is providing us with a bare string.
+   * Assume we added single quotes, then consider this case:
+   *
+   *   gsettings set x.y.z key "i'd expect this to work"
+   *
+   * A similar example could be given for double quotes.
+   *
+   * Avoid that whole mess by just using g_variant_new_string().
    */
   if (new == NULL &&
-      g_error_matches (error, G_VARIANT_PARSE_ERROR,
-                       G_VARIANT_PARSE_ERROR_UNKNOWN_KEYWORD))
+      g_variant_type_equal (type, G_VARIANT_TYPE_STRING) &&
+      value[0] != '\'' && value[0] != '"')
     {
-      value = freeme = g_strdup_printf ("\"%s\"", value);
-      new = g_variant_parse (type, value, NULL, NULL, NULL);
-      if (new != NULL)
-        g_clear_error (&error);
+      g_clear_error (&error);
+      new = g_variant_new_string (value);
     }
 
   if (new == NULL)
@@ -491,23 +502,16 @@ gsettings_set (GSettings   *settings,
   if (!g_settings_range_check (settings, key, new))
     {
       g_printerr (_("The provided value is outside of the valid range\n"));
+      g_variant_unref (new);
       exit (1);
     }
 
   g_settings_set_value (settings, key, new);
-  g_settings_sync ();
-
-  stored = g_settings_get_value (settings, key);
-  if (g_variant_equal (stored, existing))
-    {
-      g_printerr (_("Failed to set value\n"));
-      exit (1);
-    }
-
-  g_variant_unref (stored);
   g_variant_unref (existing);
   g_variant_unref (new);
 
+  g_settings_sync ();
+
   g_free (freeme);
 }
 
