@@ -1661,18 +1661,6 @@ G_CONST_RETURN gchar* g_type_name_from_instance	(GTypeInstance	*instance);
 G_CONST_RETURN gchar* g_type_name_from_class	(GTypeClass	*g_class);
 
 
-/* --- internal functions --- */
-G_GNUC_INTERNAL void    g_value_c_init          (void); /* sync with gvalue.c */
-G_GNUC_INTERNAL void    g_value_types_init      (void); /* sync with gvaluetypes.c */
-G_GNUC_INTERNAL void    g_enum_types_init       (void); /* sync with genums.c */
-G_GNUC_INTERNAL void    g_param_type_init       (void); /* sync with gparam.c */
-G_GNUC_INTERNAL void    g_boxed_type_init       (void); /* sync with gboxed.c */
-G_GNUC_INTERNAL void    g_object_type_init      (void); /* sync with gobject.c */
-G_GNUC_INTERNAL void    g_param_spec_types_init (void); /* sync with gparamspecs.c */
-G_GNUC_INTERNAL void    g_value_transforms_init (void); /* sync with gvaluetransform.c */
-G_GNUC_INTERNAL void    g_signal_init           (void); /* sync with gsignal.c */
-
-
 /* --- implementation bits --- */
 #ifndef G_DISABLE_CAST_CHECKS
 #  define _G_TYPE_CIC(ip, gt, ct) \
