@@ -22,6 +22,7 @@
 #include <string.h>
 
 #include "gvalue.h"
+#include "gtype-private.h"
 #include "genums.h"
 
 
@@ -243,7 +244,7 @@ value_transform_flags_string (const GValue *src_value,
 /* registration
  */
 void
-g_value_transforms_init (void)
+_g_value_transforms_init (void)
 {
   /* some transformations are a bit questionable,
    * we currently skip those
